from time import sleep

from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec
from selenium.common.exceptions import TimeoutException, NoSuchElementException

from stores.data_store_interface import IDataStore
from adapters.data_collection_adapter_interface import IDataCollectionAdapter
from utils.file_helper import FileHelper



class CcgpDataCollectionAdapter(IDataCollectionAdapter):
    """
    中国政府采购网数据采集适配器
    """
    file_helper = FileHelper()

    def __init__(self, url: str,store:IDataStore=None):
        self._url = url
        self._store = store
        self._driver = None
        self._keyword = None
        self._adapter_type = "ccgp"

    def login(self, username: str, password: str) -> None:
       pass

    def collect(self, keyword: str, store: IDataStore):
        if store:
            self._store = store
        self._keyword = keyword
        items = self._search(keyword)
        self._process_list(items)
        if self.config.get_bool(self.batch_save_key):
            self.store.save_collect_data(True)

    def _search(self, keyword: str) -> list:
        try:
            if not keyword:
                raise Exception("搜索关键字不能为空")
            wait = WebDriverWait(self.driver, 10, 1)
            wait.until(
                ec.presence_of_element_located((By.ID, "searchForm")))
            search_el = self.driver.find_element(By.ID, "kw")
            sleep(2)
            search_el.clear()
            search_el.send_keys(keyword)
            search_btn = self.driver.find_element(
                By.XPATH, "//form[@id='searchForm']/input[@id='doSearch2']")
            sleep(1)
            search_btn.click()
            wait.until(ec.presence_of_element_located((By.CLASS_NAME, "vT-srch-result")))
            default_search_txt = "近一周"
            search_txt = self.config.get(self.search_day_key, default_search_txt)
            self.logger.info(f"搜索关键字: {keyword}，搜索条件: {search_txt}")
            if search_txt != default_search_txt:
                last_els = self.driver.find_elements(By.XPATH, "//ul[@id='datesel']/li")
                for last_el in last_els:
                    if search_txt == last_el.text:
                        sleep(1)
                        last_el.click()
                        break
                wait.until(ec.presence_of_element_located((By.CLASS_NAME, "vT-srch-result")))
            else:
                sleep(1)


            # try:
            #     a_links = self.driver.find_elements(
            #         By.XPATH, "//form[@id='pagerSubmitForm']/a")
            #     count = len(a_links)
            #     if count > 1:
            #         count = count - 1
            #     self.logger.info(f"共查询到 {count} 页")
            # except Exception as e:
            #     self.logger.error(f"搜索失败[尝试查询页数]: {e}")
            items = self.driver.find_elements(By.XPATH,
                                         "//ul[@class='vT-srch-result-list-bid']/li/a")
            return items
        except TimeoutException as e:
            raise Exception(f"搜索失败 [超时]: {e}")
        except NoSuchElementException as e:
            raise Exception(f"搜索失败 [找不到元素]: {e}")


    def _process_list(self,  items: list) -> list:
        if not items:
            return []
        for item in items:
            self._process_item( item)
        sleep(2)
        next_items = self._next_page()
        return self._process_list( next_items)


    def _next_page(self) -> list:
        try:
            wait = WebDriverWait(self.driver, 10, 1)
            next_path = "//div[@class='vT-srch-result-list']/p/a[@class='next']"
            wait.until(ec.presence_of_element_located((By.XPATH, next_path)))
            btn = self.driver.find_element(By.XPATH, next_path)
            btn.click()
            self.logger.info(f"跳转到下页: {self.driver.current_url}")
            sleep(5)
            wait.until(ec.presence_of_element_located((By.ID, "vT-srch-result")))
            items = self.driver.find_elements(By.XPATH,
                                         "//ul[@class='vT-srch-result-list-bid']/li/a")
            return items
        except NoSuchElementException as e:
            raise Exception(f"翻页失败 [找不到元素]: {e}")
        except TimeoutException:
            self.logger.info("翻页结束")
            return []

    def _process_item(self,  item):
        main_handle = self.driver.current_window_handle
        wait = WebDriverWait(self.driver, 10, 1)
        close = True
        try:
            url = item.get_attribute('href')
            if self._check_is_collect_by_url(url):
                close = False
                return
            self.logger.info(f"跳转详情")
            sleep(1)
            item.click()
            wait.until(ec.number_of_windows_to_be(2))
            handles = self.driver.window_handles
            for handle in handles:
                if handle != main_handle:
                    self.driver.switch_to.window(handle)
                    break
            wait.until(ec.presence_of_element_located((By.TAG_NAME, "body")))
            # 判断是否为投标公告
            if self._check_type("中标公告") or  self._check_type("成交公告") or self._check_type("终止公告"):
                self._save_db(url, "", is_invalid=True)
                return
            content = self.driver.find_element(By.XPATH, "//div[@class='vF_deail_maincontent']").text
            if self._check_content(content):
                paths = []

                attach_els = self.driver.find_elements(By.XPATH, "//td[@class='bid_attachtab_content']/a")
                attach_2_els = self.driver.find_elements(By.XPATH, "//a[@ignore='1']")

                # 合并两个列表
                all_attachments = attach_els + attach_2_els
                attach_urls = []
                if len(all_attachments) > 0:
                    for attach_el in attach_els:
                        attach_url = attach_el.get_attribute('href')
                        if attach_url not in attach_urls:
                            attach_urls.append(attach_url)
                        else:
                            self.logger.info(f"重复附件: {attach_url}")
                            continue
                        file_name =  attach_el.text or attach_el.get_attribute('download') or attach_url.split('/')[-1]
                        if not file_name:
                            continue
                        # 检查 file_name 是否包含文件扩展名
                        if '.' not in file_name:
                            self.logger.warning(f"文件名 {file_name} 不包含扩展名，跳过下载。")
                            continue
                        path = self.file_helper.download_remote_file(attach_url, file_name)
                        if path:
                            paths.append(path)
                attach_str = ",".join(paths)
                self._save_db(url, content, attach_str)
            else:
                self._save_db(url, content, is_invalid=True)
        except TimeoutException as e:
            self.logger.error(
                f"采集发生异常 Timeout: {self.driver.current_url}。Exception: {e}")
        except NoSuchElementException as e:
            self.logger.error(
                f"采集发生异常 NoSuchElement: {self.driver.current_url}。Exception: {e}")
            raise Exception(f"采集失败 [找不到元素]: {e}")
        finally:
            if close:
                sleep(1)
                self.driver.close()
                self.driver.switch_to.window(main_handle)

    def _check_type(self,type_str: str)->bool:
        links = self.driver.find_elements(By.LINK_TEXT, type_str)
        if len(links) > 0:
            self.logger.info(f"{type_str}，跳过")
            return True
        return False

