from time import sleep


from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec
from selenium.common.exceptions import TimeoutException, NoSuchElementException

from stores.data_store_interface import IDataStore
from adapters.data_collection_adapter_interface import IDataCollectionAdapter


class ChinabiddingDataCollectionAdapter(IDataCollectionAdapter):
    """
    中国招标网数据采集适配器
    """


    def __init__(self, url: str,store:IDataStore=None):
        self._url = url
        self._store = store
        self._driver = None
        self._keyword = None
        self._adapter_type = "chinabidding"

    # @property
    # def store(self) -> IDataStore:
    #     return self._store
    #
    # @property
    # def url(self):
    #     return self._url
    #
    # @property
    # def keyword(self):
    #     return self._keyword
    #
    # @property
    # def driver(self)->webdriver:
    #     if not self._driver:
    #         self._driver = self._create_driver()
    #     return self._driver


    def login(self, username: str, password: str) -> None:
        try:
            login_el = self.driver.find_element(
                By.XPATH, "//div[@id='loginRight']/a[@class='login']")
            login_el.click()
            wait = WebDriverWait(self.driver, 10, 1)
            wait.until(ec.presence_of_element_located((By.ID, "userpass")))
            un_el = self.driver.find_element(By.ID, "username")
            un_el.send_keys(username)
            pass_el = self.driver.find_element(By.ID, "userpass")
            pass_el.send_keys(password)
            login_btn = self.driver.find_element(By.ID, "login-button")
            login_btn.click()
            wait.until(ec.presence_of_element_located((By.ID, "site-content")))
        except TimeoutException as e:
            raise Exception(f"登录失败 [超时]: {e}")
        except NoSuchElementException as e:
            raise Exception(f"登录失败 [找不到元素]: {e}")


    def collect(self, keyword: str, store: IDataStore):
        if store:
            self._store = store
        self._keyword = keyword
        items = self._search(keyword)
        self._process_list(items)
        if self.config.get_bool(self.batch_save_key):
            self.store.save_collect_data(True)

    def _search(self, keyword: str) -> list:
        try:
            wait = WebDriverWait(self.driver, 10, 1)
            wait.until(
                ec.presence_of_element_located((By.ID, "projSearchForm")))
            search_el = self.driver.find_element(By.ID, "fullText")
            search_el.send_keys("")
            search_el.send_keys(keyword)
            search_btn = self.driver.find_element(
                By.XPATH, "//form[@id='projSearchForm']/button")
            search_btn.click()
            wait.until(ec.presence_of_element_located((By.ID, "site-content")))
            default_search_txt = "近3日"
            search_txt = self.config.get(self.search_day_key, default_search_txt)
            self.logger.info(f"搜索关键字: {keyword}，搜索条件: {search_txt}")
            if search_txt != default_search_txt:
                last_el = self.driver.find_element(By.LINK_TEXT, search_txt)
                last_el.click()
                wait.until(ec.presence_of_element_located((By.ID, "site-content")))
            else:
                sleep(1)
            try:
                a_links = self.driver.find_elements(
                    By.XPATH, "//form[@id='pagerSubmitForm']/a")
                count = len(a_links)
                if count > 1:
                    count = count - 1
                self.logger.info(f"共查询到 {count} 页")
            except Exception as e:
                self.logger.error(f"搜索失败[尝试查询页数]: {e}")
            items = self.driver.find_elements(By.XPATH,
                                              "//ul[@class='as-pager-body']/li/a")
            return items
        except TimeoutException as e:
            raise Exception(f"搜索失败 [超时]: {e}")
        except NoSuchElementException as e:
            raise Exception(f"搜索失败 [找不到元素]: {e}")

    def _process_list(self, items: list) -> list:
        if not items:
            return []
        for item in items:
            self._process_item(item)
        sleep(2)
        next_items = self._next_page()
        return self._process_list(next_items)

    def _next_page(self) -> list:
        try:
            wait = WebDriverWait(self.driver, 10, 1)
            next_path = "//form[@id='pagerSubmitForm']/a[@class='next']"
            wait.until(ec.presence_of_element_located((By.XPATH, next_path)))
            btn = self.driver.find_element(By.XPATH, next_path)
            btn.click()
            self.logger.info(f"跳转到下页: {self.driver.current_url}")
            wait.until(ec.presence_of_element_located((By.ID, "site-content")))
            items = self.driver.find_elements(By.XPATH,
                                         "//ul[@class='as-pager-body']/li/a")
            return items
        except NoSuchElementException as e:
            raise Exception(f"翻页失败 [找不到元素]: {e}")
        except TimeoutException:
            self.logger.info("翻页结束")
            return []

    def _process_item(self, item):
        main_handle = self.driver.current_window_handle
        close = True
        try:
            url = item.get_attribute('href')
            if self._check_is_collect_by_url(url):
                close = False
                return
            item.click()
            wait = WebDriverWait(self.driver, 10, 1)
            wait.until(ec.number_of_windows_to_be(2))
            handles = self.driver.window_handles
            for handle in handles:
                if handle != main_handle:
                    self.driver.switch_to.window(handle)
                    break
            url = self.driver.current_url
            self.logger.info(f"跳转详情")
            wait.until(ec.presence_of_element_located((By.TAG_NAME, "body")))
            content = self.driver.find_element(By.TAG_NAME, "body").text
            if self._check_content(content):
                self._save_db(url, content)
            else:
                self._save_db(url, content, is_invalid=True)

        except TimeoutException as e:
            self.logger.error(
                f"采集发生异常 Timeout: {self.driver.current_url}。Exception: {e}")
            # raise Exception(f"采集失败 [超时]: {e}")
        except NoSuchElementException as e:
            self.logger.error(
                f"采集发生异常 NoSuchElement: {self.driver.current_url}。Exception: {e}")
            raise Exception(f"采集失败 [找不到元素]: {e}")
        finally:
            if close:
                sleep(2)
                self.driver.close()
                self.driver.switch_to.window(main_handle)



