from selenium import webdriver

from utils.logger_helper import LoggerHelper
from utils.config_helper import ConfigHelper


class DriverCreator:

    logger = LoggerHelper.get_logger()

    default_remote_driver_url = "http://127.0.0.1:4444/wd/hub"

    def gen_remote_driver(self, url):
        # 设置Chrome选项
        options = webdriver.ChromeOptions()

        # options.add_argument('--headless')  # 无头模式运行
        options.add_argument('--no-sandbox')
        options.add_argument('--disable-dev-shm-usage')
        options.add_experimental_option('excludeSwitches',
                                        ['enable-automation'])
        options.add_argument('--disable-blink-features=AutomationControlled')
        options.add_argument('--disable-extensions')
        # 最大化窗口
        options.add_argument('--start-maximized')
        # 无痕浏览模式
        options.add_argument('--incognito')


        remote_driver_url = ConfigHelper().get('selenium.remote_driver_url')
        if not remote_driver_url:
            remote_driver_url = self.default_remote_driver_url
            self.logger.error(
                f"未配置远程驱动地址,使用默认地址{self.default_remote_driver_url}")
        self.logger.info(f"远程驱动地址{remote_driver_url}")

        # 创建远程浏览器驱动实例
        driver = webdriver.Remote(command_executor=remote_driver_url,
                                  options=options)
        return self._gen_driver(driver, url)

    def gen_chrome_driver(self, url):
        # 设置Chrome选项，包括隐藏Selenium特征、设置代理IP和排除或关闭一些Selenium相关开关
        options = webdriver.ChromeOptions()
        options.add_experimental_option('excludeSwitches',
                                        ['enable-automation'])
        options.add_argument('--disable-blink-features=AutomationControlled')
        options.add_argument('--disable-extensions')
        # options.add_argument('--disable-gpu')
        options.add_argument('--disable-notifications')
        # options.add_argument('--disable-popup-blocking')
        # options.add_argument('--disable-web-security')
        # options.add_argument('--ignore-certificate-errors')
        # options.add_argument('--no-sandbox')
        # 最大化窗口
        options.add_argument('--start-maximized')
        # 无痕浏览模式
        options.add_argument('--incognito')
        # options.add_argument('--user-data-dir=/dev/null')
        # options.add_argument('--proxy-server={}'.format(proxy_address + ':' + proxy_port))
        # options.add_argument('--proxy-auth={}:{}'.format(proxy_username, proxy_password))
        # options.add_experimental_option('excludeSwitches', ['enable-automation', 'useAutomationExtension'])
        # 阻止浏览器窗口自动关闭
        # options.add_experimental_option('detach', True)
        driver = webdriver.Chrome(options=options)  # 创建Chrome浏览器驱动实例
        return self._gen_driver(driver, url)


    def _gen_driver(self, driver, url):
        # 设置user-agent，改变user-agent的值
        if hasattr(driver, 'execute_cdp_cmd'):
            # 隐藏navigator.webdriver标志，将其值修改为false或undefined
            driver.execute_cdp_cmd(
                'Page.addScriptToEvaluateOnNewDocument', {
                    'source':
                    'Object.defineProperty(navigator, "webdriver", {get: () => undefined})'
                })

            user_agents = [
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36",
                "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Firefox/98.0",
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Edge/98.0.1108.62 Chrome/98.0.4758.102 Safari/537.36 Edg/98.0.1108.62",
                "Mozilla/5.0 (iPad; CPU OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/14.0 Mobile/15E148 Safari/604.1",
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36 OPR/84.0.4140.129",
                "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Edge/98.0.1108.48 Chrome/98.0.4758.102 Safari/537.36 EdgA/98.0.1108.48",
            ]

            user_agent = user_agents[len(url) % len(user_agents)]
            # 设置user-agent，改变user-agent的值
            driver.execute_cdp_cmd("Network.setUserAgentOverride",
                                   {"userAgent": user_agent})
        else:
            self.logger.warning("当前驱动不支持 execute_cdp_cmd 方法")
        # url 去除空字符串
        url = url.strip()
        driver.get(url)

        # 设置隐式等待 5s
        driver.implicitly_wait(5)
        self.logger.info(f"创建浏览器驱动,URL: {url}")
        return driver


    # def shutdown_driver(self,driver):
    #     driver.quit()
